# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Alexis Sukrieh
# - Antoine Hulin
package FDN::Common::Modules;
##############################################################
# Initialisation du module
##############################################################

use strict;
use Exporter ;
our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
	conduse
	run_with_eval
	use_module
	get_foreign_function
);

use FDN::Common::Logger;
use FDN::Common::Constantes;

#
#   conduse : permet de faire l'quivalent de use mais localement.
#
#   Exemple d'utilisation :
#
#	unless (conduse('FDN::Config') {
#		error "Impossible de charger FDN::Config, j'abandonne";
#		return 0;
#	}
#
#	#  partir d'ici, on peut utiliser les fonctions de FDN::Config :
#	# elles ont t importes dans l'espace de nommage courant.
#
sub conduse ($;$) {
    my ($package, $tag) = @_;

    no strict "refs";
    if (eval "require $package") {
	if (defined $tag) {
	    $tag =~ s/^://g;
	    $package->export_to_level(1, undef,
	    	@{${$package."::EXPORT_TAGS"}{$tag}});
	} else {
	    $package->export_to_level(1, undef,
	    	(@{$package."::EXPORT_OK"}, @{$package."::EXPORT"}));
	}

	use strict;
	return TRUE;
    }

    use strict;
    error "[conduse] Package `$package' failed to import ($@)!";
    return FALSE;
}


# ----------------------------------------------------------------------
# nom           : use_base_module
# description   : essaie d'inclure le module Plutar de la sitalibs 
#                 de la base dcrite par le fichier de conf
# parametres    : le bom de la base telle que FDN::BASE::Plutar existe
# retours       : TRUE si tou va bien FALSE sinon
# ----------------------------------------------------------------------
sub use_module ($) {
        my $base = shift;
        if (run_with_eval ("use FDN::${base}::Plutar")) {
                debug "inclusion de la lib Plutar de la base '$base' russie";
                return TRUE;
        } else {  
                error "impossible d'inclure le module de la base '$base'";
                return FALSE;
        }
}

# ----------------------------------------------------------------------
# nom           : run_with_eval
# description   : lance une fonction avec eval et log une erreur si 
#                 le lancement via eval choue
# parametres    : la commande perl  lancer telle que fournie a eval
# retours       : le retour de eval
# ----------------------------------------------------------------------
sub run_with_eval ($) {
        my $commande = shift;
        return FALSE unless defined $commande;

        my $ret = eval ($commande);

        if ($@) {
                error("message d'erreur d'eval lors de l'appel '$commande' => '$@'");
                return FALSE;
        }

        if ($ret or $commande =~ /^\s*use\s+/) {
                debug "succs de l'appel de `$commande`";
                return TRUE;
        } elsif (! defined $ret) {
                warning("aucun retour de eval, a creuser ($commande)");
                return TRUE;
        } else {  
                error("chec de l'appel de `$commande` ($ret)");
                return FALSE;
        }
}

sub get_foreign_function($) {
	my ( $fct ) = @_;
	return undef if ( ! $fct );
	my $mod = $fct;
	if ( $mod =~ /::/ ) {
		$mod =~ s/::[^:]+$//;
		conduse($mod);
	}
	my $res = \&{$fct};
	return $res;
}

1;
